<?php 
// Nagios XI Operations Center Component
//
// Copyright (c) 2010 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: nocscreen.inc.php 115 2010-08-16 16:15:26Z mguthrie $

//include the helper file
require_once(dirname(__FILE__).'/../componenthelper.inc.php');

// respect the name
$nocscreen_component_name="nocscreen";

// run the initialization function
nocscreen_component_init();

nocscreen_register_dashlets(); 

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////

function nocscreen_component_init(){
	global $nocscreen_component_name;
	
	//boolean to check for latest version
	$versionok=nocscreen_component_checkversion();
	
	//component description
	$desc=gettext("This component adds a read-only NOC screen for all current unhandled problems to the home menu. 
		For larger environments an active polling interval of 60 seconds or more is recommended.");
	
	if(!$versionok)
		$desc="<b>Error: This component requires Nagios Fusion 2012R1.3 or later.</b>";
	
	//all components require a few arguments to be initialized correctly.  
	$args=array(

		// need a name
		COMPONENT_NAME => $nocscreen_component_name,
		COMPONENT_VERSION => '1.2', 
		COMPONENT_DATE => '04/04/2013',

		// informative information
		COMPONENT_AUTHOR => "Mike Guthrie. Nagios Enterprises, LLC",
		COMPONENT_DESCRIPTION => $desc,
		COMPONENT_TITLE => "Operations Center",

		// configuration function (optional)
		COMPONENT_CONFIGFUNCTION => "nocscreen_component_config_func",
		);
	
	//register this component with XI 
	register_component($nocscreen_component_name,$args);
			
	nocscreen_component_db_init($args[COMPONENT_VERSION]); 	
	
	
	//register callbacks
	if($versionok) {
		register_callback(CALLBACK_MENUS_INITIALIZED,'nocscreen_component_addmenu');
		register_callback(CALLBACK_POLLING_FUNCTIONS,'nocscreen_poller_get_noc_data'); 
		if(function_exists('register_callback'))
			register_callback(CALLBACK_DELETE_SERVER,'nocscreen_delete_server');
	}	
			
}
	
function nocscreen_component_config_func($mode="",$inargs,&$outargs,&$result){

	// initialize return code and output
	$result=0;
	$output="";
	
	$component_name="nocscreen";
	
	switch($mode){
		case COMPONENT_CONFIGMODE_GETSETTINGSHTML:
		
			//$theme=empty(get_option("nocscreen_component_theme")) ? '' : get_option("nocscreen_component_theme");

			
			// values passed to us
			$theme=grab_array_var($inargs,"theme",get_option("nocscreen_component_theme"));

			$output='
			
	<div class="sectionTitle">'.gettext('NOC Screen Component Theme').'</div>
	
	<table>

	<tr>
	<td valign="top">
	<label for="enabled">'.gettext('Theme options').':</label><br class="nobr" />
	</td>
	<td>
	<select name="theme">
		<option value="" '.is_selected($theme,'').'>Default</option>
		<option value="hc" '.is_selected($theme,'hc').'>'.gettext('High Contrast').'</option>
	</select>
	</td>
	</tr>
	</table>

			';
			break;
			
		case COMPONENT_CONFIGMODE_SAVESETTINGS:
		
			// get variables
			$theme=grab_array_var($inargs,"theme","");
			set_option("nocscreen_component_theme",$theme);
						
			break;
		
		default:
			break;
			
		}
		
	return $output;
	}


///////////////////////////////////////////////////////////////////////////////////////////
// MISC FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function nocscreen_component_checkversion(){

	if(!function_exists('get_product_release'))
		return false;
	//requires greater than 2009R1.2
	if(get_product_release()<200)
		return false;

	return true;
	}

/**
*	adds the Operations Center menu item to the Home Menu
*/ 	
function nocscreen_component_addmenu($arg=null){
	global $nocscreen_component_name;
	//retrieve the URL for this component
	$urlbase=get_component_url_base($nocscreen_component_name);

	//add this to the main home menu 
	add_menu_item(MENU_HOME,array(
		"type" => "link",
		"title" => gettext("Operations Center"),
		"id" => "menu-home-nocscreen",
		"order" => 310,
		"opts" => array(
			//this is the page the menu will actually point to.
			//all of my actual component workings will happen on this script
			"href" => $urlbase."/noc.php",      
			)
		));

}
		
/**
*	initialize xi_incidents database table if it's not there
*	@param int $im_current_version: version number of this component
*/ 		
function nocscreen_component_db_init($version) {
	global $db_tables; 
	
	db_connect_all();  
	if(!isset($db_tables[DB_NAGIOSFUSION]["options"]))
		generate_table_name(DB_NAGIOSFUSION,"options");
		
	$opt = get_option('noc_component_version'); 

	$noc_component_version = is_null($opt) ? 0 : $opt; 

	if($noc_component_version < 100) {

		//foreach($rs as $row) {
			//if($row['count']==0) {
				//create sequence
				$sql="
					CREATE SEQUENCE fusion_noc_id_seq
						INCREMENT BY 1
						NO MAXVALUE
						NO MINVALUE
						CACHE 1";
				exec_sql_query(DB_NAGIOSFUSION,$sql,true);
				//create table 
				$sql = "
					CREATE TABLE fusion_noc (
						id integer DEFAULT nextval('fusion_noc_id_seq'::regclass) NOT NULL,
						server_sid varchar(32) NOT NULL,
						type smallint DEFAULT 0, 
						host varchar(96),
						service varchar(96),
						status_update_time timestamp without time zone NOT NULL,
						status smallint DEFAULT 0,	
						output varchar(255)
					)";

					exec_sql_query(DB_NAGIOSFUSION,$sql,true);
					//make sure we own it 
					$sql="ALTER TABLE public.fusion_Noc OWNER TO nagiosfusion";	
					exec_sql_query(DB_NAGIOSFUSION,$sql,true);
			//}//end IF xi_incidents table exists 	

			set_option('noc_component_version',100); 
		//}//end foreach loop 	
	}
}	

/**
*	registers a dashlet for NOC Summary, Open Host problems, and Open Service Problems
*/ 
function nocscreen_register_dashlets () {

	//nocscreen summary
	$args=array(); 
	$args[DASHLET_NAME]="nocscreen_summary";
	$args[DASHLET_TITLE]="NOC Screen Summary";
	$args[DASHLET_FUNCTION]="nocscreen_summary";
	$args[DASHLET_DESCRIPTION]=gettext("Displays summary information for unhandled problems");
	$args[DASHLET_WIDTH]="600";
	$args[DASHLET_HEIGHT]="450";
	$args[DASHLET_INBOARD_CLASS]="noscreen_dashlet_inboard";
	$args[DASHLET_OUTBOARD_CLASS]="noscreen_dashlet_outboard";
	$args[DASHLET_CLASS]="noscreen_dashlet";
	$args[DASHLET_AUTHOR]="Mike Guthrie. Nagios Enterprises, LLC";
	$args[DASHLET_COPYRIGHT]="Dashlet Copyright &copy; 2011 Nagios Enterprises. All rights reserved.";
	$args[DASHLET_HOMEPAGE]="http://www.nagios.com";
	//$args[DASHLET_SHOWASAVAILABLE]=false;
	register_dashlet($args[DASHLET_NAME],$args);

	//nocscreen_host
	$args[DASHLET_NAME]="nocscreen_host";
	$args[DASHLET_FUNCTION]="nocscreen_host";
	$args[DASHLET_TITLE]="Unhandled Host Problems";
	$args[DASHLET_DESCRIPTION]=gettext("Displays unhandled host problems across all Nagios XI servers. <strong>(Nagios XI Only)</strong>");
	register_dashlet($args[DASHLET_NAME],$args);

	//nocscreen_service
	$args[DASHLET_NAME]="nocscreen_service";
	$args[DASHLET_FUNCTION]="nocscreen_service";
	$args[DASHLET_TITLE]="Unhandled Service Problems";
	$args[DASHLET_DESCRIPTION]=gettext("Displays all open service problems across all Nagios XI servers. <strong>(Nagios XI Only)</strong>");
	register_dashlet($args[DASHLET_NAME],$args);
}

/**
*	Server deletion callback to clean out NOC data for deleted servers
*	@param string $cbtype: the callbac type constant
*	@param mixed $args: argument array->should contain a server ID string
*/ 
function nocscreen_delete_server($cbtype='',$args=array()) {
	
	$server_id = grab_array_var($args,'server_id',false);
	if($server_id) {	
		$query = "DELETE FROM fusion_noc WHERE server_sid='{$server_id}'"; 
		exec_sql_query(DB_NAGIOSFUSION,$query,true);
	}
}

/**
*	callback function used by subsystem poller. Runs nocfetch.php as an external script. 
*	@param string $cbtype: callback type
*	@param mixed $args: args data passed from poller.php
*	@TODO: maybe turn this into a subsystem command to be more asyncronous
*/ 
function nocscreen_poller_get_noc_data($cbtype,$args) {
	global $cfg; 
		
	$servers = get_servers(); 
	$log='/usr/local/nagiosfusion/var/components/noc.log'; 
	$script='/usr/local/nagiosfusion/html/includes/components/nocscreen/nocfetch.php'; 
	
	if(filesize($log) > 100000)
		exec('echo "" > '.$log); 
	
	foreach($servers as $sid=> $server) {
		if($server['type']!='nagiosxi') continue; 
		
		$cmd="{$script} host {$sid} >> {$log}"; 
		echo "\nCMD: $cmd\n"; 
		echo exec($cmd); 
		
		$cmd="{$script} service {$sid} >> {$log}";
		echo  "\nCMD $cmd\n"; 
		exec($cmd); 
	}
	
}


/**
*	Wrapper functions for dashlet function 
*/ 
function nocscreen_host($mode) {
	if($mode==DASHLET_MODE_PREVIEW)
		return "<img src='".get_base_url()."/includes/components/nocscreen/host.png' />"; 
	return nocscreen_ajax_get_table('host'); 
}

function nocscreen_service($mode) {
	if($mode==DASHLET_MODE_PREVIEW)
		return "<img src='".get_base_url()."/includes/components/nocscreen/service.png' />"; 
	return nocscreen_ajax_get_table('service'); 
}

function nocscreen_summary($mode) {
	if($mode==DASHLET_MODE_PREVIEW)
		return "<img src='".get_base_url()."/includes/components/nocscreen/summary.png' />";		
	return nocscreen_ajax_get_table('summary'); 
}


/**
*	Javascript builder function to load Noc dashlets via ajax
*	@param string $type: host | service | summmary
*	@return string $output: html output
*/ 
function nocscreen_ajax_get_table($type) {

	$args=array(); 
	if($type=='host')
		$func='nocscreen_host_table'; 
	elseif($type=='service')
		$func='nocscreen_service_table';
	elseif($type='summary')
		$func='nocscreen_summary_table'; 
	else 
		return; 
				
	$id="nocscreen_dashlet_".random_string(6);
	
	$output='';

	// ajax updater args
	$ajaxargs=$args;
	// build args for javascript
	$n=0;
	$jargs="{";
	foreach($ajaxargs as $var => $val){
		if($n>0)
			$jargs.=", ";
		$jargs.="\"$var\" : \"$val\"";
		$n++;
		}
	$jargs.="}";

	$output.='
	<div class="nocscreen_dashlet" id="'.$id.'">
	<img src="'.theme_image("throbber.gif").'">
	</div><!--nocscreen_dashlet-->

	<script type="text/javascript">
	$(document).ready(function(){

		get_'.$id.'_content();
			
		$("#'.$id.'").everyTime(30*1000, "timer-'.$id.'", function(i) {
			get_'.$id.'_content();
		});
		
		function get_'.$id.'_content(){
			$("#'.$id.'").each(function(){
				var optsarr = {
					"func": "'.$func.'",
					"args": '.$jargs.'
					}
				var opts=array2json(optsarr);
				get_ajax_data_innerHTML("getfusioncoreajax",opts,true,this);
				});
			}		
	});
	</script>';

	return $output; 

}

/**
*	html display function for NOC summary dashlet
*/ 
function fusioncore_ajax_nocscreen_summary_table() {
	//service data 
	list($warning,$critical,$unknown,$sall) = noc_service_summary_data();
	//host data 
	list($down,$unreachable,$hall) = noc_host_summary_data(); 
	
	$table = "
	<div class='infotable_title'>".gettext("Open Problems Summary")."</div>
	<div id='summary' class='summary'>
	<table class='standardtable service_status_summary_dashlet host_status_summary_dashlet' id='summarytable'>
	  <tr class='strong'>
	    <th></th>
		<th>".gettext("Down")."</th>
		<th>".gettext("Unreachable")."</th>
		<th>".gettext("All")."</th>

		<!-- service th -->
	    <th>&nbsp;</th>
		<th>".gettext("Warning")."</th>
		<th>".gettext("Critical")."</th>
		<th>".gettext("Unknown")."</th>
		<th>".gettext("All")."</th>			
	  </tr>
	  
	  <tr class='hostrow'>
	    <td class='strong'> ".gettext("Hosts").": </td> 
		<td class='hostdown havehostdown'>{$down}</td>
		<td class='serviceunknown haveserviceunknown'>{$unreachable}</td>
		<td class='all hostproblems havehostproblems'>{$hall}</td>
		
		<!-- service stuff -->
	    <td class='strong'> ".gettext("Services").": </td>
		<td class='servicewarning haveservicewarning'>{$warning}</td>
		<td class='servicecritical haveservicecritical'>{$critical}</td>
		<td class='serviceunknown haveserviceunknown'>{$unknown}</td>
		<td class='all serviceproblems haveserviceproblems'>{$sall}</td>
	  </tr> <!-- end services row -->	  
	</table>  <!-- end summary table --> 
	</div> <!-- end summmary div -->\n";


	return $table;
}

/**
*	html display function for NOC host problems dashlet
*/ 
function fusioncore_ajax_nocscreen_host_table()
{
	$hosts = noc_get_host_problems();
	$servers=get_servers(); 
	$count = 0;	
	$table='<div class="infotable_title">'.gettext('Open Host Problems').'</div>	'.gettext('Last Update Time').': '.date('r');
	$table .="
	<div id='hostdiv'>
	  <table id='hosttable' class='standardtable hoststatustable'>
			<tr><th>".gettext("Server")."</th><th>".gettext("Host Name")."</th><th>".gettext("Last Update Time")."</th><th>".gettext("Status Information")."</th></tr>
	";
	//build table rows 	
	foreach($hosts as $x) {
		noc_host_row($table,$x,$count++,$servers);
	}
	
	//close table
	$table .="</table>\n</div>"; 
	
	return $table;
}

/**
*	html display function for NOC service problems dashlet
*/
function fusioncore_ajax_nocscreen_service_table()
{
	$hosts = noc_get_service_problems();	
	$servers=get_servers(); 
	$count = 0;	
	$table='<div class="infotable_title">'.gettext('Open Service Problems').'</div> '.gettext('Last Update Time').': '.date('r');
	$table .="
	<div id='servicediv'>
	  <table id='servicetable' class='standardtable servicestatustable'>
			<tr><th>".gettext("Server")."</th><th>".gettext("Host Name")."</th><th>".gettext("Service")."</th><th>
				<div class='duration'>".gettext("Last Update Time")."</div></th><th>".gettext("Status Information")."</th></tr>
	";
	//build table rows 	
	foreach($hosts as $x) {
		noc_service_row($table,$x,$count++,$servers); 	
	}
	//close table
	$table .="</table>\n
	".gettext("Last Update Time").": ".date('r')."</div>"; 
	
	return $table;
}

/**
*	stub function to build a formatted noc table row
*	@param string $table: REFERENCE variable for main html table string
*	@param mixed $x: the host data array / row from the dbtable
*	@param int $count: the number of rows we're on
*	@param mixed $servers: the array of server data, used to display the server's name instead of id
*/ 
function noc_host_row(&$table,$x,$count,$servers) 
{
	//builds onto main table variable 
	$class = ($count % 2 == 1) ? 'even' : 'odd'; 
	$sid = $x['server_sid']; 

	$table .= 
	"<tr class='hostrow {$class}'>
		<td class='server'>{$servers[$sid]['name']}</td>
		<td class='host_name ".noc_host_class($x['status'])."'>{$x['host']}</td>\n		
		<td><div class='duration'>{$x['status_update_time']}</div></td>\n
		<td class='statustext'>{$x['output']}</td>\n 
	</tr>"; 

}

/**
*	stub function to build a formatted noc table row
*	@param string $table: REFERENCE variable for main html table string
*	@param mixed $x: the host data array / row from the dbtable
*	@param int $count: the number of rows we're on
*	@param mixed $servers: the array of server data, used to display the server's name instead of id
*/ 
function noc_service_row(&$table,$x,$count,$servers) 
{
	//builds onto main table variable 
	$class = ($count % 2 == 1) ? 'even' : 'odd'; 
	$sid = $x['server_sid']; 
	//array_dump($x); 

	$table .= 
		"<tr class='servicerow {$class}'>
			<td class='server'>{$servers[$sid]['name']}</td>
			<td class='host_name ".noc_host_class(0)."'>{$x['host']}</td>\n
			<td class='description ".noc_service_class($x['status'])."'>{$x['service']}</td>\n
			<td><div class='duration'>{$x['status_update_time']}</div></td>\n
			<td class='statustext'>{$x['output']}</td>\n
		</tr>"; 

}


function noc_host_class($code){
	switch($code) {
		case 0:   	return "hostup";
		case 1:		return 'hostdown';
		default:	return 'hostunreachable';
	}
}

function noc_service_class($code){
	switch($code) {
		case 0:		return "serviceok";
		case 1:		return 'servicewarning';
		case 2:		return 'servicecritical';
		default:	return 'serviceunknown';
	}
}


/**
*	SQL data fetch for host problem data
*/ 
function noc_get_host_problems() {
	$sortby=grab_request_var('sortby','status_update_time'); 
	$order=grab_request_var('order','DESC');

	$query="SELECT * FROM fusion_noc WHERE type=0 ORDER BY {$sortby} {$order}"; 
	$rs=exec_sql_query(DB_NAGIOSFUSION,$query,true);
	
	if(!$rs)
		return array(); 
	else
		return $rs; 
}

/**
*	SQL data fetch for service problem data
*/ 
function noc_get_service_problems() {
	$sortby=grab_request_var('sortby','status_update_time'); 
	$order=grab_request_var('order','DESC');

	$query="SELECT * FROM fusion_noc WHERE type=1 ORDER BY {$sortby} {$order}"; 
	$rs=exec_sql_query(DB_NAGIOSFUSION,$query,true);
	
	if(!$rs)
		return array(); 
	else
		return $rs; 
}

/**
*	fetches total unhandle problem counts for nocscreen summary dashlet
*	@return mixed array: counts by state
*/ 
function noc_service_summary_data() {

	$totalwarn="SELECT COUNT(*) FROM fusion_noc WHERE type=1 AND status='1'";
	$totalcrit="SELECT COUNT(*) FROM fusion_noc WHERE type=1 AND status='2'";
	$totalunknown="SELECT COUNT(*) FROM fusion_noc WHERE type=1 AND status='3'";
	$query="SELECT ({$totalwarn}) as warning, ({$totalcrit}) as critical, ({$totalunknown}) as unknown FROM fusion_noc";
	$rs=exec_sql_query(DB_NAGIOSFUSION,$query,true); 
	//array_dump($rs); 
	if(!$rs)
		return array(0,0,0,0); 
		
	foreach($rs as $r)
		return array($r['warning'],$r['critical'],$r['unknown'],($r['warning']+$r['critical']+$r['unknown'])); 

}

/**
*	fetches total unhandle problem counts for nocscreen summary dashlet
*	@return mixed array: counts by state
*/ 
function noc_host_summary_data() {

	$totaldown="SELECT COUNT(*) FROM fusion_noc WHERE type=0 AND status='1'";
	$totalunreachable="SELECT COUNT(*) FROM fusion_noc WHERE type=0 AND status='2'";
	$query="SELECT ({$totaldown}) as down, ({$totalunreachable}) as unreachable, ({$totalunreachable}) as unreachable FROM fusion_noc";
	$rs=exec_sql_query(DB_NAGIOSFUSION,$query,true); 
	//array_dump($rs); 
	if(!$rs)
		return array(0,0,0); 
	
	foreach($rs as $r)
		return array($r['down'],$r['unreachable'],($r['down']+$r['unreachable'])); 
		
}


